# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AccountObject,
        ApproverObject,
        ApproverObjectApprover,
        ApproverObjectPolicy,
        BatchProgressResponse,
        BatchProgressResponseStatus,
        BatchResultsResponse,
        BatchResultsResponseResultsItem,
        BulkInvoiceFailure,
        BulkInvoiceOperationResponse,
        BulkInvoiceOperationResponseFailedItem,
        BulkInvoiceUnexportOperationResponse,
        BulkInvoiceUnexportOperationResponseFailedItem,
        BulkInvoiceUnexportOperationResponseSkippedItem,
        BulkUpsertAsyncResponse,
        BulkUpsertAsyncResponseSummary,
        CatalogAllowanceBulkItem,
        CatalogAllowanceBulkItemAllowanceType,
        CatalogAllowanceListResponse,
        CatalogAllowanceListResponseResultsItem,
        CatalogAllowanceListResponseResultsItemAllowanceType,
        CatalogAllowanceListResponseResultsItemTargetType,
        CatalogAllowanceNestedCreateRequest,
        CatalogAllowanceNestedCreateRequestAllowanceType,
        CatalogAllowanceNestedObject,
        CatalogAllowanceNestedObjectAllowanceType,
        CatalogAllowanceNestedObjectTargetType,
        CatalogAllowanceObject,
        CatalogAllowanceObjectAllowanceType,
        CatalogAllowanceObjectTargetType,
        CatalogBulkErrorItem,
        CatalogBulkResponse,
        CatalogBulkResponseErrorsItem,
        CatalogBulkResponseSkippedItem,
        CatalogBulkSkippedItem,
        CatalogEntryBasicObject,
        CatalogEntryBulkItem,
        CatalogEntryBulkItemAllowancesItem,
        CatalogEntryBulkItemAllowancesItemAllowanceType,
        CatalogEntryBulkItemPricesItem,
        CatalogEntryBulkItemPricesItemPriceType,
        CatalogEntryCreateRequest,
        CatalogEntryCreateRequestAllowancesItem,
        CatalogEntryCreateRequestAllowancesItemAllowanceType,
        CatalogEntryCreateRequestPricesItem,
        CatalogEntryCreateRequestPricesItemPriceType,
        CatalogEntryListResponse,
        CatalogEntryListResponseResultsItem,
        CatalogEntryObject,
        CatalogEntryObjectAllowancesItem,
        CatalogEntryObjectAllowancesItemAllowanceType,
        CatalogEntryObjectAllowancesItemTargetType,
        CatalogEntryObjectPricesItem,
        CatalogEntryObjectPricesItemPriceTypeDisplay,
        CatalogErrorResponse,
        CatalogPriceCreateRequest,
        CatalogPriceCreateRequestPriceType,
        CatalogPriceItem,
        CatalogPriceItemPriceTypeDisplay,
        CompanyObject,
        ErrorResponse,
        InstantCaptureFileObject,
        InvoiceCreationResponse,
        InvoiceCreationResponseBatch,
        InvoiceCreationResponseBatchStatus,
        InvoiceCreationResponseContainer,
        InvoiceCreationResponseInvoice,
        InvoiceCreationResponseInvoiceStatus,
        InvoiceCreationResponseProcessing,
        InvoiceCreationResponseProcessingStatus,
        InvoiceCreationResponseUpload,
        InvoiceCreationResponseUploadUploadMethod,
        InvoiceDetailObject,
        InvoiceDetailObjectGlSplitsItem,
        InvoiceDetailObjectHistoryItem,
        InvoiceDetailObjectHistoryItemUser,
        InvoiceDetailObjectImagesItem,
        InvoiceDetailObjectItemsProcessingResult,
        InvoiceDetailObjectItemsProcessingResultStatus,
        InvoiceDetailObjectPaymentInfo,
        InvoiceDetailObjectPaymentStatus,
        InvoiceDetailObjectType,
        InvoiceItemVarianceListResponse,
        InvoiceItemVarianceListResponseResultsItem,
        InvoiceNeedsAttentionObject,
        InvoiceNeedsAttentionObjectOnHold,
        InvoiceNeedsAttentionObjectPaymentStatus,
        InvoiceNeedsAttentionObjectType,
        InvoiceObject,
        InvoiceObjectImagesItem,
        InvoiceObjectOnHold,
        InvoiceObjectPaymentStatus,
        InvoiceObjectType,
        LineItemObject,
        LocationObject,
        OauthScope,
        PurchaseOrderCreateResponse,
        PurchaseOrderCreateResponseItemsItem,
        PurchaseOrderCreateResponseStatus,
        PurchaseOrderDetail,
        PurchaseOrderDetailPurchaseOrder,
        PurchaseOrderDetailPurchaseOrderItemsItem,
        PurchaseOrderDetailPurchaseOrderStatus,
        PurchaseOrderSummary,
        PurchaseOrderSummaryStatus,
        PurchaseOrderUpdateResponse,
        PurchaseOrderUpdateResponseStatus,
        ReceiptBulkUpsertAsyncResponse,
        ReceiptBulkUpsertAsyncResponseSummary,
        ReceiptCreateResponse,
        ReceiptCreateResponseStatus,
        ReceiptDetail,
        ReceiptDetailReceipt,
        ReceiptDetailReceiptItemsItem,
        ReceiptDetailReceiptStatus,
        ReceiptItem,
        ReceiptSummary,
        ReceiptSummaryStatus,
        ReceiptUpdateResponse,
        ReceiptUpdateResponseStatus,
        VarianceListResponse,
        VarianceListResponseResultsItem,
        VarianceListResponseResultsItemLastInvoiceItem,
        VarianceObject,
        VarianceObjectInvoiceItems,
        VarianceObjectInvoiceItemsDataItem,
        VarianceObjectLastInvoiceItem,
    )
    from .errors import (
        BadRequestError,
        ContentTooLargeError,
        ForbiddenError,
        InternalServerError,
        NotFoundError,
        PreconditionFailedError,
        UnauthorizedError,
        UnprocessableEntityError,
    )
    from . import accounts, batch, catalog, dimensions, invoices, oauth, purchase_orders, receipts, vendors
    from .accounts import GetAccountsIdCompaniesResponse, GetAccountsIdLocationsResponse, GetAccountsRootResponse
    from .batch import (
        GetBatchRootRequestStatus,
        GetBatchRootResponse,
        GetBatchRootResponseResultsItem,
        GetBatchRootResponseResultsItemStatus,
    )
    from .catalog import (
        CatalogAllowanceBulkCreateRequestAllowancesItem,
        CatalogAllowanceBulkCreateRequestAllowancesItemAllowanceType,
        CatalogEntryBulkCreateRequestEntriesItem,
        CatalogEntryBulkCreateRequestEntriesItemAllowancesItem,
        CatalogEntryBulkCreateRequestEntriesItemAllowancesItemAllowanceType,
        CatalogEntryBulkCreateRequestEntriesItemPricesItem,
        CatalogEntryBulkCreateRequestEntriesItemPricesItemPriceType,
        CatalogPricesBulkCreateRequestPricesItem,
        CatalogPricesBulkCreateRequestPricesItemPriceType,
        GetCatalogPricesRequestPriceType,
        GetCatalogPricesResponse,
        GetCatalogVariancesRequestExceptionType,
        PatchCatalogAllowancesIdRequestAllowanceType,
        PatchCatalogEntriesIdRequestProperties,
        PostCatalogAllowancesRequestAllowanceType,
        PostCatalogEntriesRequestAllowancesItem,
        PostCatalogEntriesRequestPricesItem,
        PostCatalogEntriesRequestPricesItemPriceType,
        PostCatalogPricesBulkResponse,
        PostCatalogPricesRequestPriceType,
    )
    from .client import AsyncOttimate, Ottimate
    from .dimensions import (
        GetDimensionsIdResponse,
        GetDimensionsIdResponseDimension,
        GetDimensionsIdResponseDimensionOttimateType,
        GetDimensionsRootResponse,
        GetDimensionsRootResponseDimensionsItem,
        GetDimensionsRootResponseDimensionsItemOttimateType,
        PostDimensionsBulkRequestDimensionsItem,
        PostDimensionsBulkRequestDimensionsItemOttimateType,
        PostDimensionsBulkResponse,
        PostDimensionsBulkResponseCreatedItem,
        PostDimensionsBulkResponseErrorsItem,
        PostDimensionsBulkResponseUpdatedItem,
        PostDimensionsRootRequestOttimateType,
        PostDimensionsRootResponse,
        PostDimensionsRootResponseOttimateType,
    )
    from .environment import OttimateEnvironment
    from .invoices import (
        DeleteInvoicesIdApproversResponse,
        GetInvoicesIdApproversResponse,
        GetInvoicesRootRequestPaymentStatus,
        GetInvoicesRootRequestStatus,
        GetInvoicesRootResponse,
        GetInvoicesUploadsResponse,
        PatchInvoicesIdRequestItemsItem,
        PatchInvoicesIdRequestType,
        PostInvoicesIdApproveResponse,
        PostInvoicesIdApproveResponseInvoice,
        PostInvoicesIdApproversResponse,
        PostInvoicesIdFlagResponse,
        PostInvoicesIdFlagResponseInvoice,
        PostInvoicesMarkExportedRequestInvoicesItem,
        PostInvoicesMarkUnexportedRequestInvoicesItem,
        PostInvoicesRootRequestItemsItem,
        PostInvoicesRootRequestType,
        PostInvoicesRootResponse,
        PostInvoicesRootResponseItems,
    )
    from .oauth import PostOauthTokenResponse
    from .purchase_orders import (
        GetPurchaseOrdersRootRequestStatus,
        GetPurchaseOrdersRootResponse,
        PatchPurchaseOrdersIdRequestItemsItem,
        PurchaseOrderBulkUpsertRequestPurchaseOrdersItem,
        PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem,
        PurchaseOrderCreateRequestItemsItem,
    )
    from .receipts import (
        GetReceiptsRootResponse,
        PatchReceiptsIdRequestItemsItem,
        ReceiptBulkUpsertRequestReceiptsItem,
        ReceiptBulkUpsertRequestReceiptsItemItemsItem,
        ReceiptCreateRequestItemsItem,
    )
    from .vendors import (
        GetVendorsIdResponse,
        GetVendorsIdResponseVendor,
        GetVendorsIdResponseVendorPaymentMethod,
        GetVendorsRootResponse,
        GetVendorsRootResponseVendorsItem,
        GetVendorsRootResponseVendorsItemPaymentMethod,
        PostVendorsBulkRequestVendorsItem,
        PostVendorsBulkRequestVendorsItemPaymentMethod,
        PostVendorsBulkResponse,
        PostVendorsBulkResponseCreatedItem,
        PostVendorsBulkResponseCreatedItemPaymentMethod,
        PostVendorsBulkResponseErrorsItem,
        PostVendorsBulkResponseUpdatedItem,
        PostVendorsBulkResponseUpdatedItemPaymentMethod,
        PostVendorsRootRequestPaymentMethod,
        PostVendorsRootResponse,
        PostVendorsRootResponseVendor,
        PostVendorsRootResponseVendorPaymentMethod,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AccountObject": ".types",
    "ApproverObject": ".types",
    "ApproverObjectApprover": ".types",
    "ApproverObjectPolicy": ".types",
    "AsyncOttimate": ".client",
    "BadRequestError": ".errors",
    "BatchProgressResponse": ".types",
    "BatchProgressResponseStatus": ".types",
    "BatchResultsResponse": ".types",
    "BatchResultsResponseResultsItem": ".types",
    "BulkInvoiceFailure": ".types",
    "BulkInvoiceOperationResponse": ".types",
    "BulkInvoiceOperationResponseFailedItem": ".types",
    "BulkInvoiceUnexportOperationResponse": ".types",
    "BulkInvoiceUnexportOperationResponseFailedItem": ".types",
    "BulkInvoiceUnexportOperationResponseSkippedItem": ".types",
    "BulkUpsertAsyncResponse": ".types",
    "BulkUpsertAsyncResponseSummary": ".types",
    "CatalogAllowanceBulkCreateRequestAllowancesItem": ".catalog",
    "CatalogAllowanceBulkCreateRequestAllowancesItemAllowanceType": ".catalog",
    "CatalogAllowanceBulkItem": ".types",
    "CatalogAllowanceBulkItemAllowanceType": ".types",
    "CatalogAllowanceListResponse": ".types",
    "CatalogAllowanceListResponseResultsItem": ".types",
    "CatalogAllowanceListResponseResultsItemAllowanceType": ".types",
    "CatalogAllowanceListResponseResultsItemTargetType": ".types",
    "CatalogAllowanceNestedCreateRequest": ".types",
    "CatalogAllowanceNestedCreateRequestAllowanceType": ".types",
    "CatalogAllowanceNestedObject": ".types",
    "CatalogAllowanceNestedObjectAllowanceType": ".types",
    "CatalogAllowanceNestedObjectTargetType": ".types",
    "CatalogAllowanceObject": ".types",
    "CatalogAllowanceObjectAllowanceType": ".types",
    "CatalogAllowanceObjectTargetType": ".types",
    "CatalogBulkErrorItem": ".types",
    "CatalogBulkResponse": ".types",
    "CatalogBulkResponseErrorsItem": ".types",
    "CatalogBulkResponseSkippedItem": ".types",
    "CatalogBulkSkippedItem": ".types",
    "CatalogEntryBasicObject": ".types",
    "CatalogEntryBulkCreateRequestEntriesItem": ".catalog",
    "CatalogEntryBulkCreateRequestEntriesItemAllowancesItem": ".catalog",
    "CatalogEntryBulkCreateRequestEntriesItemAllowancesItemAllowanceType": ".catalog",
    "CatalogEntryBulkCreateRequestEntriesItemPricesItem": ".catalog",
    "CatalogEntryBulkCreateRequestEntriesItemPricesItemPriceType": ".catalog",
    "CatalogEntryBulkItem": ".types",
    "CatalogEntryBulkItemAllowancesItem": ".types",
    "CatalogEntryBulkItemAllowancesItemAllowanceType": ".types",
    "CatalogEntryBulkItemPricesItem": ".types",
    "CatalogEntryBulkItemPricesItemPriceType": ".types",
    "CatalogEntryCreateRequest": ".types",
    "CatalogEntryCreateRequestAllowancesItem": ".types",
    "CatalogEntryCreateRequestAllowancesItemAllowanceType": ".types",
    "CatalogEntryCreateRequestPricesItem": ".types",
    "CatalogEntryCreateRequestPricesItemPriceType": ".types",
    "CatalogEntryListResponse": ".types",
    "CatalogEntryListResponseResultsItem": ".types",
    "CatalogEntryObject": ".types",
    "CatalogEntryObjectAllowancesItem": ".types",
    "CatalogEntryObjectAllowancesItemAllowanceType": ".types",
    "CatalogEntryObjectAllowancesItemTargetType": ".types",
    "CatalogEntryObjectPricesItem": ".types",
    "CatalogEntryObjectPricesItemPriceTypeDisplay": ".types",
    "CatalogErrorResponse": ".types",
    "CatalogPriceCreateRequest": ".types",
    "CatalogPriceCreateRequestPriceType": ".types",
    "CatalogPriceItem": ".types",
    "CatalogPriceItemPriceTypeDisplay": ".types",
    "CatalogPricesBulkCreateRequestPricesItem": ".catalog",
    "CatalogPricesBulkCreateRequestPricesItemPriceType": ".catalog",
    "CompanyObject": ".types",
    "ContentTooLargeError": ".errors",
    "DeleteInvoicesIdApproversResponse": ".invoices",
    "ErrorResponse": ".types",
    "ForbiddenError": ".errors",
    "GetAccountsIdCompaniesResponse": ".accounts",
    "GetAccountsIdLocationsResponse": ".accounts",
    "GetAccountsRootResponse": ".accounts",
    "GetBatchRootRequestStatus": ".batch",
    "GetBatchRootResponse": ".batch",
    "GetBatchRootResponseResultsItem": ".batch",
    "GetBatchRootResponseResultsItemStatus": ".batch",
    "GetCatalogPricesRequestPriceType": ".catalog",
    "GetCatalogPricesResponse": ".catalog",
    "GetCatalogVariancesRequestExceptionType": ".catalog",
    "GetDimensionsIdResponse": ".dimensions",
    "GetDimensionsIdResponseDimension": ".dimensions",
    "GetDimensionsIdResponseDimensionOttimateType": ".dimensions",
    "GetDimensionsRootResponse": ".dimensions",
    "GetDimensionsRootResponseDimensionsItem": ".dimensions",
    "GetDimensionsRootResponseDimensionsItemOttimateType": ".dimensions",
    "GetInvoicesIdApproversResponse": ".invoices",
    "GetInvoicesRootRequestPaymentStatus": ".invoices",
    "GetInvoicesRootRequestStatus": ".invoices",
    "GetInvoicesRootResponse": ".invoices",
    "GetInvoicesUploadsResponse": ".invoices",
    "GetPurchaseOrdersRootRequestStatus": ".purchase_orders",
    "GetPurchaseOrdersRootResponse": ".purchase_orders",
    "GetReceiptsRootResponse": ".receipts",
    "GetVendorsIdResponse": ".vendors",
    "GetVendorsIdResponseVendor": ".vendors",
    "GetVendorsIdResponseVendorPaymentMethod": ".vendors",
    "GetVendorsRootResponse": ".vendors",
    "GetVendorsRootResponseVendorsItem": ".vendors",
    "GetVendorsRootResponseVendorsItemPaymentMethod": ".vendors",
    "InstantCaptureFileObject": ".types",
    "InternalServerError": ".errors",
    "InvoiceCreationResponse": ".types",
    "InvoiceCreationResponseBatch": ".types",
    "InvoiceCreationResponseBatchStatus": ".types",
    "InvoiceCreationResponseContainer": ".types",
    "InvoiceCreationResponseInvoice": ".types",
    "InvoiceCreationResponseInvoiceStatus": ".types",
    "InvoiceCreationResponseProcessing": ".types",
    "InvoiceCreationResponseProcessingStatus": ".types",
    "InvoiceCreationResponseUpload": ".types",
    "InvoiceCreationResponseUploadUploadMethod": ".types",
    "InvoiceDetailObject": ".types",
    "InvoiceDetailObjectGlSplitsItem": ".types",
    "InvoiceDetailObjectHistoryItem": ".types",
    "InvoiceDetailObjectHistoryItemUser": ".types",
    "InvoiceDetailObjectImagesItem": ".types",
    "InvoiceDetailObjectItemsProcessingResult": ".types",
    "InvoiceDetailObjectItemsProcessingResultStatus": ".types",
    "InvoiceDetailObjectPaymentInfo": ".types",
    "InvoiceDetailObjectPaymentStatus": ".types",
    "InvoiceDetailObjectType": ".types",
    "InvoiceItemVarianceListResponse": ".types",
    "InvoiceItemVarianceListResponseResultsItem": ".types",
    "InvoiceNeedsAttentionObject": ".types",
    "InvoiceNeedsAttentionObjectOnHold": ".types",
    "InvoiceNeedsAttentionObjectPaymentStatus": ".types",
    "InvoiceNeedsAttentionObjectType": ".types",
    "InvoiceObject": ".types",
    "InvoiceObjectImagesItem": ".types",
    "InvoiceObjectOnHold": ".types",
    "InvoiceObjectPaymentStatus": ".types",
    "InvoiceObjectType": ".types",
    "LineItemObject": ".types",
    "LocationObject": ".types",
    "NotFoundError": ".errors",
    "OauthScope": ".types",
    "Ottimate": ".client",
    "OttimateEnvironment": ".environment",
    "PatchCatalogAllowancesIdRequestAllowanceType": ".catalog",
    "PatchCatalogEntriesIdRequestProperties": ".catalog",
    "PatchInvoicesIdRequestItemsItem": ".invoices",
    "PatchInvoicesIdRequestType": ".invoices",
    "PatchPurchaseOrdersIdRequestItemsItem": ".purchase_orders",
    "PatchReceiptsIdRequestItemsItem": ".receipts",
    "PostCatalogAllowancesRequestAllowanceType": ".catalog",
    "PostCatalogEntriesRequestAllowancesItem": ".catalog",
    "PostCatalogEntriesRequestPricesItem": ".catalog",
    "PostCatalogEntriesRequestPricesItemPriceType": ".catalog",
    "PostCatalogPricesBulkResponse": ".catalog",
    "PostCatalogPricesRequestPriceType": ".catalog",
    "PostDimensionsBulkRequestDimensionsItem": ".dimensions",
    "PostDimensionsBulkRequestDimensionsItemOttimateType": ".dimensions",
    "PostDimensionsBulkResponse": ".dimensions",
    "PostDimensionsBulkResponseCreatedItem": ".dimensions",
    "PostDimensionsBulkResponseErrorsItem": ".dimensions",
    "PostDimensionsBulkResponseUpdatedItem": ".dimensions",
    "PostDimensionsRootRequestOttimateType": ".dimensions",
    "PostDimensionsRootResponse": ".dimensions",
    "PostDimensionsRootResponseOttimateType": ".dimensions",
    "PostInvoicesIdApproveResponse": ".invoices",
    "PostInvoicesIdApproveResponseInvoice": ".invoices",
    "PostInvoicesIdApproversResponse": ".invoices",
    "PostInvoicesIdFlagResponse": ".invoices",
    "PostInvoicesIdFlagResponseInvoice": ".invoices",
    "PostInvoicesMarkExportedRequestInvoicesItem": ".invoices",
    "PostInvoicesMarkUnexportedRequestInvoicesItem": ".invoices",
    "PostInvoicesRootRequestItemsItem": ".invoices",
    "PostInvoicesRootRequestType": ".invoices",
    "PostInvoicesRootResponse": ".invoices",
    "PostInvoicesRootResponseItems": ".invoices",
    "PostOauthTokenResponse": ".oauth",
    "PostVendorsBulkRequestVendorsItem": ".vendors",
    "PostVendorsBulkRequestVendorsItemPaymentMethod": ".vendors",
    "PostVendorsBulkResponse": ".vendors",
    "PostVendorsBulkResponseCreatedItem": ".vendors",
    "PostVendorsBulkResponseCreatedItemPaymentMethod": ".vendors",
    "PostVendorsBulkResponseErrorsItem": ".vendors",
    "PostVendorsBulkResponseUpdatedItem": ".vendors",
    "PostVendorsBulkResponseUpdatedItemPaymentMethod": ".vendors",
    "PostVendorsRootRequestPaymentMethod": ".vendors",
    "PostVendorsRootResponse": ".vendors",
    "PostVendorsRootResponseVendor": ".vendors",
    "PostVendorsRootResponseVendorPaymentMethod": ".vendors",
    "PreconditionFailedError": ".errors",
    "PurchaseOrderBulkUpsertRequestPurchaseOrdersItem": ".purchase_orders",
    "PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem": ".purchase_orders",
    "PurchaseOrderCreateRequestItemsItem": ".purchase_orders",
    "PurchaseOrderCreateResponse": ".types",
    "PurchaseOrderCreateResponseItemsItem": ".types",
    "PurchaseOrderCreateResponseStatus": ".types",
    "PurchaseOrderDetail": ".types",
    "PurchaseOrderDetailPurchaseOrder": ".types",
    "PurchaseOrderDetailPurchaseOrderItemsItem": ".types",
    "PurchaseOrderDetailPurchaseOrderStatus": ".types",
    "PurchaseOrderSummary": ".types",
    "PurchaseOrderSummaryStatus": ".types",
    "PurchaseOrderUpdateResponse": ".types",
    "PurchaseOrderUpdateResponseStatus": ".types",
    "ReceiptBulkUpsertAsyncResponse": ".types",
    "ReceiptBulkUpsertAsyncResponseSummary": ".types",
    "ReceiptBulkUpsertRequestReceiptsItem": ".receipts",
    "ReceiptBulkUpsertRequestReceiptsItemItemsItem": ".receipts",
    "ReceiptCreateRequestItemsItem": ".receipts",
    "ReceiptCreateResponse": ".types",
    "ReceiptCreateResponseStatus": ".types",
    "ReceiptDetail": ".types",
    "ReceiptDetailReceipt": ".types",
    "ReceiptDetailReceiptItemsItem": ".types",
    "ReceiptDetailReceiptStatus": ".types",
    "ReceiptItem": ".types",
    "ReceiptSummary": ".types",
    "ReceiptSummaryStatus": ".types",
    "ReceiptUpdateResponse": ".types",
    "ReceiptUpdateResponseStatus": ".types",
    "UnauthorizedError": ".errors",
    "UnprocessableEntityError": ".errors",
    "VarianceListResponse": ".types",
    "VarianceListResponseResultsItem": ".types",
    "VarianceListResponseResultsItemLastInvoiceItem": ".types",
    "VarianceObject": ".types",
    "VarianceObjectInvoiceItems": ".types",
    "VarianceObjectInvoiceItemsDataItem": ".types",
    "VarianceObjectLastInvoiceItem": ".types",
    "accounts": ".accounts",
    "batch": ".batch",
    "catalog": ".catalog",
    "dimensions": ".dimensions",
    "invoices": ".invoices",
    "oauth": ".oauth",
    "purchase_orders": ".purchase_orders",
    "receipts": ".receipts",
    "vendors": ".vendors",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AccountObject",
    "ApproverObject",
    "ApproverObjectApprover",
    "ApproverObjectPolicy",
    "AsyncOttimate",
    "BadRequestError",
    "BatchProgressResponse",
    "BatchProgressResponseStatus",
    "BatchResultsResponse",
    "BatchResultsResponseResultsItem",
    "BulkInvoiceFailure",
    "BulkInvoiceOperationResponse",
    "BulkInvoiceOperationResponseFailedItem",
    "BulkInvoiceUnexportOperationResponse",
    "BulkInvoiceUnexportOperationResponseFailedItem",
    "BulkInvoiceUnexportOperationResponseSkippedItem",
    "BulkUpsertAsyncResponse",
    "BulkUpsertAsyncResponseSummary",
    "CatalogAllowanceBulkCreateRequestAllowancesItem",
    "CatalogAllowanceBulkCreateRequestAllowancesItemAllowanceType",
    "CatalogAllowanceBulkItem",
    "CatalogAllowanceBulkItemAllowanceType",
    "CatalogAllowanceListResponse",
    "CatalogAllowanceListResponseResultsItem",
    "CatalogAllowanceListResponseResultsItemAllowanceType",
    "CatalogAllowanceListResponseResultsItemTargetType",
    "CatalogAllowanceNestedCreateRequest",
    "CatalogAllowanceNestedCreateRequestAllowanceType",
    "CatalogAllowanceNestedObject",
    "CatalogAllowanceNestedObjectAllowanceType",
    "CatalogAllowanceNestedObjectTargetType",
    "CatalogAllowanceObject",
    "CatalogAllowanceObjectAllowanceType",
    "CatalogAllowanceObjectTargetType",
    "CatalogBulkErrorItem",
    "CatalogBulkResponse",
    "CatalogBulkResponseErrorsItem",
    "CatalogBulkResponseSkippedItem",
    "CatalogBulkSkippedItem",
    "CatalogEntryBasicObject",
    "CatalogEntryBulkCreateRequestEntriesItem",
    "CatalogEntryBulkCreateRequestEntriesItemAllowancesItem",
    "CatalogEntryBulkCreateRequestEntriesItemAllowancesItemAllowanceType",
    "CatalogEntryBulkCreateRequestEntriesItemPricesItem",
    "CatalogEntryBulkCreateRequestEntriesItemPricesItemPriceType",
    "CatalogEntryBulkItem",
    "CatalogEntryBulkItemAllowancesItem",
    "CatalogEntryBulkItemAllowancesItemAllowanceType",
    "CatalogEntryBulkItemPricesItem",
    "CatalogEntryBulkItemPricesItemPriceType",
    "CatalogEntryCreateRequest",
    "CatalogEntryCreateRequestAllowancesItem",
    "CatalogEntryCreateRequestAllowancesItemAllowanceType",
    "CatalogEntryCreateRequestPricesItem",
    "CatalogEntryCreateRequestPricesItemPriceType",
    "CatalogEntryListResponse",
    "CatalogEntryListResponseResultsItem",
    "CatalogEntryObject",
    "CatalogEntryObjectAllowancesItem",
    "CatalogEntryObjectAllowancesItemAllowanceType",
    "CatalogEntryObjectAllowancesItemTargetType",
    "CatalogEntryObjectPricesItem",
    "CatalogEntryObjectPricesItemPriceTypeDisplay",
    "CatalogErrorResponse",
    "CatalogPriceCreateRequest",
    "CatalogPriceCreateRequestPriceType",
    "CatalogPriceItem",
    "CatalogPriceItemPriceTypeDisplay",
    "CatalogPricesBulkCreateRequestPricesItem",
    "CatalogPricesBulkCreateRequestPricesItemPriceType",
    "CompanyObject",
    "ContentTooLargeError",
    "DeleteInvoicesIdApproversResponse",
    "ErrorResponse",
    "ForbiddenError",
    "GetAccountsIdCompaniesResponse",
    "GetAccountsIdLocationsResponse",
    "GetAccountsRootResponse",
    "GetBatchRootRequestStatus",
    "GetBatchRootResponse",
    "GetBatchRootResponseResultsItem",
    "GetBatchRootResponseResultsItemStatus",
    "GetCatalogPricesRequestPriceType",
    "GetCatalogPricesResponse",
    "GetCatalogVariancesRequestExceptionType",
    "GetDimensionsIdResponse",
    "GetDimensionsIdResponseDimension",
    "GetDimensionsIdResponseDimensionOttimateType",
    "GetDimensionsRootResponse",
    "GetDimensionsRootResponseDimensionsItem",
    "GetDimensionsRootResponseDimensionsItemOttimateType",
    "GetInvoicesIdApproversResponse",
    "GetInvoicesRootRequestPaymentStatus",
    "GetInvoicesRootRequestStatus",
    "GetInvoicesRootResponse",
    "GetInvoicesUploadsResponse",
    "GetPurchaseOrdersRootRequestStatus",
    "GetPurchaseOrdersRootResponse",
    "GetReceiptsRootResponse",
    "GetVendorsIdResponse",
    "GetVendorsIdResponseVendor",
    "GetVendorsIdResponseVendorPaymentMethod",
    "GetVendorsRootResponse",
    "GetVendorsRootResponseVendorsItem",
    "GetVendorsRootResponseVendorsItemPaymentMethod",
    "InstantCaptureFileObject",
    "InternalServerError",
    "InvoiceCreationResponse",
    "InvoiceCreationResponseBatch",
    "InvoiceCreationResponseBatchStatus",
    "InvoiceCreationResponseContainer",
    "InvoiceCreationResponseInvoice",
    "InvoiceCreationResponseInvoiceStatus",
    "InvoiceCreationResponseProcessing",
    "InvoiceCreationResponseProcessingStatus",
    "InvoiceCreationResponseUpload",
    "InvoiceCreationResponseUploadUploadMethod",
    "InvoiceDetailObject",
    "InvoiceDetailObjectGlSplitsItem",
    "InvoiceDetailObjectHistoryItem",
    "InvoiceDetailObjectHistoryItemUser",
    "InvoiceDetailObjectImagesItem",
    "InvoiceDetailObjectItemsProcessingResult",
    "InvoiceDetailObjectItemsProcessingResultStatus",
    "InvoiceDetailObjectPaymentInfo",
    "InvoiceDetailObjectPaymentStatus",
    "InvoiceDetailObjectType",
    "InvoiceItemVarianceListResponse",
    "InvoiceItemVarianceListResponseResultsItem",
    "InvoiceNeedsAttentionObject",
    "InvoiceNeedsAttentionObjectOnHold",
    "InvoiceNeedsAttentionObjectPaymentStatus",
    "InvoiceNeedsAttentionObjectType",
    "InvoiceObject",
    "InvoiceObjectImagesItem",
    "InvoiceObjectOnHold",
    "InvoiceObjectPaymentStatus",
    "InvoiceObjectType",
    "LineItemObject",
    "LocationObject",
    "NotFoundError",
    "OauthScope",
    "Ottimate",
    "OttimateEnvironment",
    "PatchCatalogAllowancesIdRequestAllowanceType",
    "PatchCatalogEntriesIdRequestProperties",
    "PatchInvoicesIdRequestItemsItem",
    "PatchInvoicesIdRequestType",
    "PatchPurchaseOrdersIdRequestItemsItem",
    "PatchReceiptsIdRequestItemsItem",
    "PostCatalogAllowancesRequestAllowanceType",
    "PostCatalogEntriesRequestAllowancesItem",
    "PostCatalogEntriesRequestPricesItem",
    "PostCatalogEntriesRequestPricesItemPriceType",
    "PostCatalogPricesBulkResponse",
    "PostCatalogPricesRequestPriceType",
    "PostDimensionsBulkRequestDimensionsItem",
    "PostDimensionsBulkRequestDimensionsItemOttimateType",
    "PostDimensionsBulkResponse",
    "PostDimensionsBulkResponseCreatedItem",
    "PostDimensionsBulkResponseErrorsItem",
    "PostDimensionsBulkResponseUpdatedItem",
    "PostDimensionsRootRequestOttimateType",
    "PostDimensionsRootResponse",
    "PostDimensionsRootResponseOttimateType",
    "PostInvoicesIdApproveResponse",
    "PostInvoicesIdApproveResponseInvoice",
    "PostInvoicesIdApproversResponse",
    "PostInvoicesIdFlagResponse",
    "PostInvoicesIdFlagResponseInvoice",
    "PostInvoicesMarkExportedRequestInvoicesItem",
    "PostInvoicesMarkUnexportedRequestInvoicesItem",
    "PostInvoicesRootRequestItemsItem",
    "PostInvoicesRootRequestType",
    "PostInvoicesRootResponse",
    "PostInvoicesRootResponseItems",
    "PostOauthTokenResponse",
    "PostVendorsBulkRequestVendorsItem",
    "PostVendorsBulkRequestVendorsItemPaymentMethod",
    "PostVendorsBulkResponse",
    "PostVendorsBulkResponseCreatedItem",
    "PostVendorsBulkResponseCreatedItemPaymentMethod",
    "PostVendorsBulkResponseErrorsItem",
    "PostVendorsBulkResponseUpdatedItem",
    "PostVendorsBulkResponseUpdatedItemPaymentMethod",
    "PostVendorsRootRequestPaymentMethod",
    "PostVendorsRootResponse",
    "PostVendorsRootResponseVendor",
    "PostVendorsRootResponseVendorPaymentMethod",
    "PreconditionFailedError",
    "PurchaseOrderBulkUpsertRequestPurchaseOrdersItem",
    "PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem",
    "PurchaseOrderCreateRequestItemsItem",
    "PurchaseOrderCreateResponse",
    "PurchaseOrderCreateResponseItemsItem",
    "PurchaseOrderCreateResponseStatus",
    "PurchaseOrderDetail",
    "PurchaseOrderDetailPurchaseOrder",
    "PurchaseOrderDetailPurchaseOrderItemsItem",
    "PurchaseOrderDetailPurchaseOrderStatus",
    "PurchaseOrderSummary",
    "PurchaseOrderSummaryStatus",
    "PurchaseOrderUpdateResponse",
    "PurchaseOrderUpdateResponseStatus",
    "ReceiptBulkUpsertAsyncResponse",
    "ReceiptBulkUpsertAsyncResponseSummary",
    "ReceiptBulkUpsertRequestReceiptsItem",
    "ReceiptBulkUpsertRequestReceiptsItemItemsItem",
    "ReceiptCreateRequestItemsItem",
    "ReceiptCreateResponse",
    "ReceiptCreateResponseStatus",
    "ReceiptDetail",
    "ReceiptDetailReceipt",
    "ReceiptDetailReceiptItemsItem",
    "ReceiptDetailReceiptStatus",
    "ReceiptItem",
    "ReceiptSummary",
    "ReceiptSummaryStatus",
    "ReceiptUpdateResponse",
    "ReceiptUpdateResponseStatus",
    "UnauthorizedError",
    "UnprocessableEntityError",
    "VarianceListResponse",
    "VarianceListResponseResultsItem",
    "VarianceListResponseResultsItemLastInvoiceItem",
    "VarianceObject",
    "VarianceObjectInvoiceItems",
    "VarianceObjectInvoiceItemsDataItem",
    "VarianceObjectLastInvoiceItem",
    "accounts",
    "batch",
    "catalog",
    "dimensions",
    "invoices",
    "oauth",
    "purchase_orders",
    "receipts",
    "vendors",
]



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceDetailObjectItemsProcessingResultStatus = typing.Union[
    typing.Literal["success", "partial_success", "failure"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .invoice_creation_response_processing_status import InvoiceCreationResponseProcessingStatus


class InvoiceCreationResponseProcessing(UniversalBaseModel):
    """
    Asynchronous processing information
    """

    status: InvoiceCreationResponseProcessingStatus = pydantic.Field()
    """
    Processing workflow status
    """

    estimated_completion: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Estimated processing completion time
    """

    queue_position: typing.Optional[int] = pydantic.Field(default=None)
    """
    Position in processing queue
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CatalogAllowanceNestedObjectAllowanceType = typing.Union[typing.Literal["allowance", "discount", "rebate"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

PurchaseOrderUpdateResponseStatus = typing.Union[
    typing.Literal["pending", "open", "received", "closed", "flagged", "archived", "deleted"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.bulk_upsert_async_response import BulkUpsertAsyncResponse
from ..types.purchase_order_create_response import PurchaseOrderCreateResponse
from ..types.purchase_order_detail import PurchaseOrderDetail
from ..types.purchase_order_update_response import PurchaseOrderUpdateResponse
from .raw_client import AsyncRawPurchaseOrdersClient, RawPurchaseOrdersClient
from .types.get_purchase_orders_root_request_status import GetPurchaseOrdersRootRequestStatus
from .types.get_purchase_orders_root_response import GetPurchaseOrdersRootResponse
from .types.patch_purchase_orders_id_request_items_item import PatchPurchaseOrdersIdRequestItemsItem
from .types.purchase_order_bulk_upsert_request_purchase_orders_item import (
    PurchaseOrderBulkUpsertRequestPurchaseOrdersItem,
)
from .types.purchase_order_create_request_items_item import PurchaseOrderCreateRequestItemsItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PurchaseOrdersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPurchaseOrdersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPurchaseOrdersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPurchaseOrdersClient
        """
        return self._raw_client

    def get_purchase_orders_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        ottimate_location_id: typing.Optional[int] = None,
        erp_vendor_id: typing.Optional[str] = None,
        status: typing.Optional[GetPurchaseOrdersRootRequestStatus] = None,
        date_from: typing.Optional[str] = None,
        date_to: typing.Optional[str] = None,
        expand: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetPurchaseOrdersRootResponse:
        """
        Returns a paginated list of purchase orders, filterable by company, location, vendor, status, and date range.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter by company ID

        ottimate_location_id : typing.Optional[int]
            Filter by location ID

        erp_vendor_id : typing.Optional[str]
            Filter by ERP vendor ID (external vendor identifier)

        status : typing.Optional[GetPurchaseOrdersRootRequestStatus]
            Filter by PO status

        date_from : typing.Optional[str]
            Filter by start date (YYYY-MM-DD)

        date_to : typing.Optional[str]
            Filter by end date (YYYY-MM-DD)

        expand : typing.Optional[str]
            Comma-separated list of fields to expand. Supported values: items. When expand=items is used, date_from and date_to must both be provided and within a 30-day window.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPurchaseOrdersRootResponse
            Successful response with paginated purchase order list

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.purchase_orders.get_purchase_orders_root(
            ottimate_company_id=15196,
            ottimate_location_id=12345,
            erp_vendor_id="VENDOR-123",
            expand="items",
        )
        """
        _response = self._raw_client.get_purchase_orders_root(
            ottimate_company_id=ottimate_company_id,
            ottimate_location_id=ottimate_location_id,
            erp_vendor_id=erp_vendor_id,
            status=status,
            date_from=date_from,
            date_to=date_to,
            expand=expand,
            request_options=request_options,
        )
        return _response.data

    def post_purchase_orders_root(
        self,
        *,
        ottimate_location_id: int,
        po_number: str,
        erp_vendor_id: str,
        date: str,
        total_amount: float,
        items: typing.Sequence[PurchaseOrderCreateRequestItemsItem],
        external_id: typing.Optional[str] = OMIT,
        erp_vendor_name: typing.Optional[str] = OMIT,
        tax: typing.Optional[float] = OMIT,
        freight: typing.Optional[float] = OMIT,
        miscellaneous_charges: typing.Optional[float] = OMIT,
        invoice_number_ref: typing.Optional[str] = OMIT,
        is2way: typing.Optional[bool] = OMIT,
        custom_fields: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PurchaseOrderCreateResponse:
        """
        Creates a new purchase order with the provided details. The `po_number` field is required and must be provided by the client (no auto-generation).

        Parameters
        ----------
        ottimate_location_id : int

        po_number : str

        erp_vendor_id : str
            External vendor identifier from ERP system

        date : str

        total_amount : float

        items : typing.Sequence[PurchaseOrderCreateRequestItemsItem]

        external_id : typing.Optional[str]

        erp_vendor_name : typing.Optional[str]
            Vendor name from ERP system

        tax : typing.Optional[float]

        freight : typing.Optional[float]

        miscellaneous_charges : typing.Optional[float]

        invoice_number_ref : typing.Optional[str]
            Invoice reference number

        is2way : typing.Optional[bool]
            Whether this is a 2-way match PO

        custom_fields : typing.Optional[typing.Dict[str, typing.Any]]
            Optional custom fields configured for this company's purchase orders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PurchaseOrderCreateResponse
            Successfully created

        Examples
        --------
        from ottimate import Ottimate
        from ottimate.purchase_orders import PurchaseOrderCreateRequestItemsItem

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.purchase_orders.post_purchase_orders_root(
            ottimate_location_id=12345,
            external_id="ERP-PO-001",
            po_number="PO-2024-001",
            erp_vendor_id="VENDOR-123",
            erp_vendor_name="Sysco Foods",
            date="2024-01-15",
            total_amount=1500.0,
            items=[
                PurchaseOrderCreateRequestItemsItem(
                    external_id="ERP-ITEM-001",
                    name="Tomatoes",
                    sku="TOM-001",
                    quantity=100.0,
                    price=2.5,
                )
            ],
            custom_fields={"TransactionId": "EXT-123456", "CustomField2": "value2"},
        )
        """
        _response = self._raw_client.post_purchase_orders_root(
            ottimate_location_id=ottimate_location_id,
            po_number=po_number,
            erp_vendor_id=erp_vendor_id,
            date=date,
            total_amount=total_amount,
            items=items,
            external_id=external_id,
            erp_vendor_name=erp_vendor_name,
            tax=tax,
            freight=freight,
            miscellaneous_charges=miscellaneous_charges,
            invoice_number_ref=invoice_number_ref,
            is2way=is2way,
            custom_fields=custom_fields,
            request_options=request_options,
        )
        return _response.data

    def get_purchase_orders_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PurchaseOrderDetail:
        """
        Returns detailed information for a single purchase order including all line items.

        Parameters
        ----------
        id : str
            The unique ID of the purchase order

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PurchaseOrderDetail
            Successful response with purchase order details

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.purchase_orders.get_purchase_orders_id(
            id="racc_90832490382904802384",
        )
        """
        _response = self._raw_client.get_purchase_orders_id(id, request_options=request_options)
        return _response.data

    def patch_purchase_orders_id(
        self,
        id: str,
        *,
        erp_vendor_name: typing.Optional[str] = OMIT,
        date: typing.Optional[str] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        tax: typing.Optional[float] = OMIT,
        freight: typing.Optional[float] = OMIT,
        miscellaneous_charges: typing.Optional[float] = OMIT,
        invoice_number_ref: typing.Optional[str] = OMIT,
        is2way: typing.Optional[bool] = OMIT,
        items: typing.Optional[typing.Sequence[PatchPurchaseOrdersIdRequestItemsItem]] = OMIT,
        custom_fields: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PurchaseOrderUpdateResponse:
        """
        Updates a purchase order. Note: `status`, `po_number`, `external_id`, `erp_vendor_id`, and `ottimate_location_id` cannot be changed.

        Parameters
        ----------
        id : str
            The unique ID of the purchase order

        erp_vendor_name : typing.Optional[str]
            Vendor name from ERP system

        date : typing.Optional[str]

        total_amount : typing.Optional[float]

        tax : typing.Optional[float]

        freight : typing.Optional[float]

        miscellaneous_charges : typing.Optional[float]

        invoice_number_ref : typing.Optional[str]

        is2way : typing.Optional[bool]

        items : typing.Optional[typing.Sequence[PatchPurchaseOrdersIdRequestItemsItem]]
            Items with `id` update the existing item. Items without `id` create a new item. When creating, `name`, `quantity`, and `price` are required.

        custom_fields : typing.Optional[typing.Dict[str, typing.Any]]
            Optional custom fields configured for this company's purchase orders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PurchaseOrderUpdateResponse
            Successfully updated

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.purchase_orders.patch_purchase_orders_id(
            id="racc_90832490382904802384",
            total_amount=1600.0,
        )
        """
        _response = self._raw_client.patch_purchase_orders_id(
            id,
            erp_vendor_name=erp_vendor_name,
            date=date,
            total_amount=total_amount,
            tax=tax,
            freight=freight,
            miscellaneous_charges=miscellaneous_charges,
            invoice_number_ref=invoice_number_ref,
            is2way=is2way,
            items=items,
            custom_fields=custom_fields,
            request_options=request_options,
        )
        return _response.data

    def post_purchase_orders_bulk(
        self,
        *,
        ottimate_company_id: int,
        ottimate_location_id: int,
        purchase_orders: typing.Sequence[PurchaseOrderBulkUpsertRequestPurchaseOrdersItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertAsyncResponse:
        """
        Creates or updates multiple purchase orders in a single request. Maximum 100 purchase orders and 1000 total line items per request. Processing is always asynchronous via Celery and returns a batch tracking ID for polling progress.

        Parameters
        ----------
        ottimate_company_id : int
            Company ID for all purchase orders in this batch

        ottimate_location_id : int
            Default location ID for all purchase orders in this batch

        purchase_orders : typing.Sequence[PurchaseOrderBulkUpsertRequestPurchaseOrdersItem]
            Max 100 POs with max 1000 total items

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertAsyncResponse
            Asynchronous processing started. Poll batch_url for progress.

        Examples
        --------
        from ottimate import Ottimate
        from ottimate.purchase_orders import (
            PurchaseOrderBulkUpsertRequestPurchaseOrdersItem,
            PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem,
        )

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.purchase_orders.post_purchase_orders_bulk(
            ottimate_company_id=15196,
            ottimate_location_id=12345,
            purchase_orders=[
                PurchaseOrderBulkUpsertRequestPurchaseOrdersItem(
                    external_id="ERP-PO-001",
                    po_number="PO-2024-001",
                    erp_vendor_id="VENDOR-123",
                    erp_vendor_name="Sysco Foods",
                    date="2024-01-15",
                    total_amount=1500.0,
                    items=[
                        PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem(
                            external_id="ERP-ITEM-001",
                            name="Tomatoes",
                            quantity=100.0,
                            price=2.5,
                        )
                    ],
                )
            ],
        )
        """
        _response = self._raw_client.post_purchase_orders_bulk(
            ottimate_company_id=ottimate_company_id,
            ottimate_location_id=ottimate_location_id,
            purchase_orders=purchase_orders,
            request_options=request_options,
        )
        return _response.data


class AsyncPurchaseOrdersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPurchaseOrdersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPurchaseOrdersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPurchaseOrdersClient
        """
        return self._raw_client

    async def get_purchase_orders_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        ottimate_location_id: typing.Optional[int] = None,
        erp_vendor_id: typing.Optional[str] = None,
        status: typing.Optional[GetPurchaseOrdersRootRequestStatus] = None,
        date_from: typing.Optional[str] = None,
        date_to: typing.Optional[str] = None,
        expand: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetPurchaseOrdersRootResponse:
        """
        Returns a paginated list of purchase orders, filterable by company, location, vendor, status, and date range.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter by company ID

        ottimate_location_id : typing.Optional[int]
            Filter by location ID

        erp_vendor_id : typing.Optional[str]
            Filter by ERP vendor ID (external vendor identifier)

        status : typing.Optional[GetPurchaseOrdersRootRequestStatus]
            Filter by PO status

        date_from : typing.Optional[str]
            Filter by start date (YYYY-MM-DD)

        date_to : typing.Optional[str]
            Filter by end date (YYYY-MM-DD)

        expand : typing.Optional[str]
            Comma-separated list of fields to expand. Supported values: items. When expand=items is used, date_from and date_to must both be provided and within a 30-day window.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPurchaseOrdersRootResponse
            Successful response with paginated purchase order list

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.purchase_orders.get_purchase_orders_root(
                ottimate_company_id=15196,
                ottimate_location_id=12345,
                erp_vendor_id="VENDOR-123",
                expand="items",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_purchase_orders_root(
            ottimate_company_id=ottimate_company_id,
            ottimate_location_id=ottimate_location_id,
            erp_vendor_id=erp_vendor_id,
            status=status,
            date_from=date_from,
            date_to=date_to,
            expand=expand,
            request_options=request_options,
        )
        return _response.data

    async def post_purchase_orders_root(
        self,
        *,
        ottimate_location_id: int,
        po_number: str,
        erp_vendor_id: str,
        date: str,
        total_amount: float,
        items: typing.Sequence[PurchaseOrderCreateRequestItemsItem],
        external_id: typing.Optional[str] = OMIT,
        erp_vendor_name: typing.Optional[str] = OMIT,
        tax: typing.Optional[float] = OMIT,
        freight: typing.Optional[float] = OMIT,
        miscellaneous_charges: typing.Optional[float] = OMIT,
        invoice_number_ref: typing.Optional[str] = OMIT,
        is2way: typing.Optional[bool] = OMIT,
        custom_fields: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PurchaseOrderCreateResponse:
        """
        Creates a new purchase order with the provided details. The `po_number` field is required and must be provided by the client (no auto-generation).

        Parameters
        ----------
        ottimate_location_id : int

        po_number : str

        erp_vendor_id : str
            External vendor identifier from ERP system

        date : str

        total_amount : float

        items : typing.Sequence[PurchaseOrderCreateRequestItemsItem]

        external_id : typing.Optional[str]

        erp_vendor_name : typing.Optional[str]
            Vendor name from ERP system

        tax : typing.Optional[float]

        freight : typing.Optional[float]

        miscellaneous_charges : typing.Optional[float]

        invoice_number_ref : typing.Optional[str]
            Invoice reference number

        is2way : typing.Optional[bool]
            Whether this is a 2-way match PO

        custom_fields : typing.Optional[typing.Dict[str, typing.Any]]
            Optional custom fields configured for this company's purchase orders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PurchaseOrderCreateResponse
            Successfully created

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate
        from ottimate.purchase_orders import PurchaseOrderCreateRequestItemsItem

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.purchase_orders.post_purchase_orders_root(
                ottimate_location_id=12345,
                external_id="ERP-PO-001",
                po_number="PO-2024-001",
                erp_vendor_id="VENDOR-123",
                erp_vendor_name="Sysco Foods",
                date="2024-01-15",
                total_amount=1500.0,
                items=[
                    PurchaseOrderCreateRequestItemsItem(
                        external_id="ERP-ITEM-001",
                        name="Tomatoes",
                        sku="TOM-001",
                        quantity=100.0,
                        price=2.5,
                    )
                ],
                custom_fields={"TransactionId": "EXT-123456", "CustomField2": "value2"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_purchase_orders_root(
            ottimate_location_id=ottimate_location_id,
            po_number=po_number,
            erp_vendor_id=erp_vendor_id,
            date=date,
            total_amount=total_amount,
            items=items,
            external_id=external_id,
            erp_vendor_name=erp_vendor_name,
            tax=tax,
            freight=freight,
            miscellaneous_charges=miscellaneous_charges,
            invoice_number_ref=invoice_number_ref,
            is2way=is2way,
            custom_fields=custom_fields,
            request_options=request_options,
        )
        return _response.data

    async def get_purchase_orders_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PurchaseOrderDetail:
        """
        Returns detailed information for a single purchase order including all line items.

        Parameters
        ----------
        id : str
            The unique ID of the purchase order

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PurchaseOrderDetail
            Successful response with purchase order details

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.purchase_orders.get_purchase_orders_id(
                id="racc_90832490382904802384",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_purchase_orders_id(id, request_options=request_options)
        return _response.data

    async def patch_purchase_orders_id(
        self,
        id: str,
        *,
        erp_vendor_name: typing.Optional[str] = OMIT,
        date: typing.Optional[str] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        tax: typing.Optional[float] = OMIT,
        freight: typing.Optional[float] = OMIT,
        miscellaneous_charges: typing.Optional[float] = OMIT,
        invoice_number_ref: typing.Optional[str] = OMIT,
        is2way: typing.Optional[bool] = OMIT,
        items: typing.Optional[typing.Sequence[PatchPurchaseOrdersIdRequestItemsItem]] = OMIT,
        custom_fields: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PurchaseOrderUpdateResponse:
        """
        Updates a purchase order. Note: `status`, `po_number`, `external_id`, `erp_vendor_id`, and `ottimate_location_id` cannot be changed.

        Parameters
        ----------
        id : str
            The unique ID of the purchase order

        erp_vendor_name : typing.Optional[str]
            Vendor name from ERP system

        date : typing.Optional[str]

        total_amount : typing.Optional[float]

        tax : typing.Optional[float]

        freight : typing.Optional[float]

        miscellaneous_charges : typing.Optional[float]

        invoice_number_ref : typing.Optional[str]

        is2way : typing.Optional[bool]

        items : typing.Optional[typing.Sequence[PatchPurchaseOrdersIdRequestItemsItem]]
            Items with `id` update the existing item. Items without `id` create a new item. When creating, `name`, `quantity`, and `price` are required.

        custom_fields : typing.Optional[typing.Dict[str, typing.Any]]
            Optional custom fields configured for this company's purchase orders

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PurchaseOrderUpdateResponse
            Successfully updated

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.purchase_orders.patch_purchase_orders_id(
                id="racc_90832490382904802384",
                total_amount=1600.0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch_purchase_orders_id(
            id,
            erp_vendor_name=erp_vendor_name,
            date=date,
            total_amount=total_amount,
            tax=tax,
            freight=freight,
            miscellaneous_charges=miscellaneous_charges,
            invoice_number_ref=invoice_number_ref,
            is2way=is2way,
            items=items,
            custom_fields=custom_fields,
            request_options=request_options,
        )
        return _response.data

    async def post_purchase_orders_bulk(
        self,
        *,
        ottimate_company_id: int,
        ottimate_location_id: int,
        purchase_orders: typing.Sequence[PurchaseOrderBulkUpsertRequestPurchaseOrdersItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BulkUpsertAsyncResponse:
        """
        Creates or updates multiple purchase orders in a single request. Maximum 100 purchase orders and 1000 total line items per request. Processing is always asynchronous via Celery and returns a batch tracking ID for polling progress.

        Parameters
        ----------
        ottimate_company_id : int
            Company ID for all purchase orders in this batch

        ottimate_location_id : int
            Default location ID for all purchase orders in this batch

        purchase_orders : typing.Sequence[PurchaseOrderBulkUpsertRequestPurchaseOrdersItem]
            Max 100 POs with max 1000 total items

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BulkUpsertAsyncResponse
            Asynchronous processing started. Poll batch_url for progress.

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate
        from ottimate.purchase_orders import (
            PurchaseOrderBulkUpsertRequestPurchaseOrdersItem,
            PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem,
        )

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.purchase_orders.post_purchase_orders_bulk(
                ottimate_company_id=15196,
                ottimate_location_id=12345,
                purchase_orders=[
                    PurchaseOrderBulkUpsertRequestPurchaseOrdersItem(
                        external_id="ERP-PO-001",
                        po_number="PO-2024-001",
                        erp_vendor_id="VENDOR-123",
                        erp_vendor_name="Sysco Foods",
                        date="2024-01-15",
                        total_amount=1500.0,
                        items=[
                            PurchaseOrderBulkUpsertRequestPurchaseOrdersItemItemsItem(
                                external_id="ERP-ITEM-001",
                                name="Tomatoes",
                                quantity=100.0,
                                price=2.5,
                            )
                        ],
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_purchase_orders_bulk(
            ottimate_company_id=ottimate_company_id,
            ottimate_location_id=ottimate_location_id,
            purchase_orders=purchase_orders,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceCreationResponseUploadUploadMethod = typing.Union[
    typing.Literal["file_upload", "url_download", "public_api"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        DeleteInvoicesIdApproversResponse,
        GetInvoicesIdApproversResponse,
        GetInvoicesRootRequestPaymentStatus,
        GetInvoicesRootRequestStatus,
        GetInvoicesRootResponse,
        GetInvoicesUploadsResponse,
        PatchInvoicesIdRequestItemsItem,
        PatchInvoicesIdRequestType,
        PostInvoicesIdApproveResponse,
        PostInvoicesIdApproveResponseInvoice,
        PostInvoicesIdApproversResponse,
        PostInvoicesIdFlagResponse,
        PostInvoicesIdFlagResponseInvoice,
        PostInvoicesMarkExportedRequestInvoicesItem,
        PostInvoicesMarkUnexportedRequestInvoicesItem,
        PostInvoicesRootRequestItemsItem,
        PostInvoicesRootRequestType,
        PostInvoicesRootResponse,
        PostInvoicesRootResponseItems,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "DeleteInvoicesIdApproversResponse": ".types",
    "GetInvoicesIdApproversResponse": ".types",
    "GetInvoicesRootRequestPaymentStatus": ".types",
    "GetInvoicesRootRequestStatus": ".types",
    "GetInvoicesRootResponse": ".types",
    "GetInvoicesUploadsResponse": ".types",
    "PatchInvoicesIdRequestItemsItem": ".types",
    "PatchInvoicesIdRequestType": ".types",
    "PostInvoicesIdApproveResponse": ".types",
    "PostInvoicesIdApproveResponseInvoice": ".types",
    "PostInvoicesIdApproversResponse": ".types",
    "PostInvoicesIdFlagResponse": ".types",
    "PostInvoicesIdFlagResponseInvoice": ".types",
    "PostInvoicesMarkExportedRequestInvoicesItem": ".types",
    "PostInvoicesMarkUnexportedRequestInvoicesItem": ".types",
    "PostInvoicesRootRequestItemsItem": ".types",
    "PostInvoicesRootRequestType": ".types",
    "PostInvoicesRootResponse": ".types",
    "PostInvoicesRootResponseItems": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DeleteInvoicesIdApproversResponse",
    "GetInvoicesIdApproversResponse",
    "GetInvoicesRootRequestPaymentStatus",
    "GetInvoicesRootRequestStatus",
    "GetInvoicesRootResponse",
    "GetInvoicesUploadsResponse",
    "PatchInvoicesIdRequestItemsItem",
    "PatchInvoicesIdRequestType",
    "PostInvoicesIdApproveResponse",
    "PostInvoicesIdApproveResponseInvoice",
    "PostInvoicesIdApproversResponse",
    "PostInvoicesIdFlagResponse",
    "PostInvoicesIdFlagResponseInvoice",
    "PostInvoicesMarkExportedRequestInvoicesItem",
    "PostInvoicesMarkUnexportedRequestInvoicesItem",
    "PostInvoicesRootRequestItemsItem",
    "PostInvoicesRootRequestType",
    "PostInvoicesRootResponse",
    "PostInvoicesRootResponseItems",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawVendorsClient, RawVendorsClient
from .types.get_vendors_id_response import GetVendorsIdResponse
from .types.get_vendors_root_response import GetVendorsRootResponse
from .types.post_vendors_bulk_request_vendors_item import PostVendorsBulkRequestVendorsItem
from .types.post_vendors_bulk_response import PostVendorsBulkResponse
from .types.post_vendors_root_request_payment_method import PostVendorsRootRequestPaymentMethod
from .types.post_vendors_root_response import PostVendorsRootResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VendorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVendorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVendorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVendorsClient
        """
        return self._raw_client

    def get_vendors_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        detail: typing.Optional[typing.Literal["basic"]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetVendorsRootResponse:
        """
        Returns a paginated list of accounting vendors, filterable by company. The `detail=basic` query parameter returns a simplified response.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter vendors by company ID

        detail : typing.Optional[typing.Literal["basic"]]
            Specify the level of detail in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVendorsRootResponse
            Successful response with paginated vendor list

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.vendors.get_vendors_root(
            ottimate_company_id=15196,
        )
        """
        _response = self._raw_client.get_vendors_root(
            ottimate_company_id=ottimate_company_id, detail=detail, request_options=request_options
        )
        return _response.data

    def post_vendors_root(
        self,
        *,
        ottimate_company_id: int,
        erp_vendor_name: str,
        erp_vendor_id: str,
        email: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        net_terms: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[PostVendorsRootRequestPaymentMethod] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zipcode: typing.Optional[str] = OMIT,
        address_line1: typing.Optional[str] = OMIT,
        address_line2: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostVendorsRootResponse:
        """
        Creates a new vendor with the provided details. The `erp_vendor_id` is required and will be mapped to both `accounting_id` and `dd_unique_key`.

        Parameters
        ----------
        ottimate_company_id : int

        erp_vendor_name : str

        erp_vendor_id : str

        email : typing.Optional[str]

        phone : typing.Optional[str]

        net_terms : typing.Optional[str]

        payment_method : typing.Optional[PostVendorsRootRequestPaymentMethod]
            (Optional) Vendor payment method

        city : typing.Optional[str]
            (Optional) Vendor city

        state : typing.Optional[str]
            (Optional) Vendor state

        zipcode : typing.Optional[str]
            (Optional) Vendor zipcode

        address_line1 : typing.Optional[str]
            (Optional) Vendor address line 1

        address_line2 : typing.Optional[str]
            (Optional) Vendor address line 2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostVendorsRootResponse
            Successfully created

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.vendors.post_vendors_root(
            ottimate_company_id=15196,
            erp_vendor_name="New Food Supplier Inc",
            erp_vendor_id="NFS-001",
            email="orders@newfoodsupplier.com",
            phone="+1-555-123-4567",
            net_terms="Net 30",
        )
        """
        _response = self._raw_client.post_vendors_root(
            ottimate_company_id=ottimate_company_id,
            erp_vendor_name=erp_vendor_name,
            erp_vendor_id=erp_vendor_id,
            email=email,
            phone=phone,
            net_terms=net_terms,
            payment_method=payment_method,
            city=city,
            state=state,
            zipcode=zipcode,
            address_line1=address_line1,
            address_line2=address_line2,
            request_options=request_options,
        )
        return _response.data

    def get_vendors_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetVendorsIdResponse:
        """
        Returns detailed information for a single accounting vendor using its unique ID.

        Parameters
        ----------
        id : str
            The unique ID of the vendor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVendorsIdResponse
            Successful response with vendor details

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.vendors.get_vendors_id(
            id="racc_90832490382904802384",
        )
        """
        _response = self._raw_client.get_vendors_id(id, request_options=request_options)
        return _response.data

    def post_vendors_bulk(
        self,
        *,
        ottimate_company_id: int,
        vendors: typing.Optional[typing.Sequence[PostVendorsBulkRequestVendorsItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostVendorsBulkResponse:
        """
        Allows for the creation or updating of multiple vendors in a single request. Maximum 100 vendors per request.

        Parameters
        ----------
        ottimate_company_id : int

        vendors : typing.Optional[typing.Sequence[PostVendorsBulkRequestVendorsItem]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostVendorsBulkResponse
            Successfully created or updated

        Examples
        --------
        from ottimate import Ottimate
        from ottimate.vendors import PostVendorsBulkRequestVendorsItem

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.vendors.post_vendors_bulk(
            ottimate_company_id=15196,
            vendors=[
                PostVendorsBulkRequestVendorsItem(
                    erp_vendor_name="Local Produce Co",
                    erp_vendor_id="LPC-001",
                    email="orders@localproduce.com",
                    phone="+1-555-111-2222",
                    net_terms="Net 15",
                    payment_method="ach",
                    address_line1="100 Farm Road",
                    city="Springfield",
                    state="IL",
                    zipcode="62701",
                ),
                PostVendorsBulkRequestVendorsItem(
                    erp_vendor_name="Premium Meats LLC",
                    erp_vendor_id="PML-001",
                    email="sales@premiummeats.com",
                    payment_method="check",
                ),
            ],
        )
        """
        _response = self._raw_client.post_vendors_bulk(
            ottimate_company_id=ottimate_company_id, vendors=vendors, request_options=request_options
        )
        return _response.data


class AsyncVendorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVendorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVendorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVendorsClient
        """
        return self._raw_client

    async def get_vendors_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        detail: typing.Optional[typing.Literal["basic"]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetVendorsRootResponse:
        """
        Returns a paginated list of accounting vendors, filterable by company. The `detail=basic` query parameter returns a simplified response.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter vendors by company ID

        detail : typing.Optional[typing.Literal["basic"]]
            Specify the level of detail in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVendorsRootResponse
            Successful response with paginated vendor list

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendors.get_vendors_root(
                ottimate_company_id=15196,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_vendors_root(
            ottimate_company_id=ottimate_company_id, detail=detail, request_options=request_options
        )
        return _response.data

    async def post_vendors_root(
        self,
        *,
        ottimate_company_id: int,
        erp_vendor_name: str,
        erp_vendor_id: str,
        email: typing.Optional[str] = OMIT,
        phone: typing.Optional[str] = OMIT,
        net_terms: typing.Optional[str] = OMIT,
        payment_method: typing.Optional[PostVendorsRootRequestPaymentMethod] = OMIT,
        city: typing.Optional[str] = OMIT,
        state: typing.Optional[str] = OMIT,
        zipcode: typing.Optional[str] = OMIT,
        address_line1: typing.Optional[str] = OMIT,
        address_line2: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostVendorsRootResponse:
        """
        Creates a new vendor with the provided details. The `erp_vendor_id` is required and will be mapped to both `accounting_id` and `dd_unique_key`.

        Parameters
        ----------
        ottimate_company_id : int

        erp_vendor_name : str

        erp_vendor_id : str

        email : typing.Optional[str]

        phone : typing.Optional[str]

        net_terms : typing.Optional[str]

        payment_method : typing.Optional[PostVendorsRootRequestPaymentMethod]
            (Optional) Vendor payment method

        city : typing.Optional[str]
            (Optional) Vendor city

        state : typing.Optional[str]
            (Optional) Vendor state

        zipcode : typing.Optional[str]
            (Optional) Vendor zipcode

        address_line1 : typing.Optional[str]
            (Optional) Vendor address line 1

        address_line2 : typing.Optional[str]
            (Optional) Vendor address line 2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostVendorsRootResponse
            Successfully created

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendors.post_vendors_root(
                ottimate_company_id=15196,
                erp_vendor_name="New Food Supplier Inc",
                erp_vendor_id="NFS-001",
                email="orders@newfoodsupplier.com",
                phone="+1-555-123-4567",
                net_terms="Net 30",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_vendors_root(
            ottimate_company_id=ottimate_company_id,
            erp_vendor_name=erp_vendor_name,
            erp_vendor_id=erp_vendor_id,
            email=email,
            phone=phone,
            net_terms=net_terms,
            payment_method=payment_method,
            city=city,
            state=state,
            zipcode=zipcode,
            address_line1=address_line1,
            address_line2=address_line2,
            request_options=request_options,
        )
        return _response.data

    async def get_vendors_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetVendorsIdResponse:
        """
        Returns detailed information for a single accounting vendor using its unique ID.

        Parameters
        ----------
        id : str
            The unique ID of the vendor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetVendorsIdResponse
            Successful response with vendor details

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendors.get_vendors_id(
                id="racc_90832490382904802384",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_vendors_id(id, request_options=request_options)
        return _response.data

    async def post_vendors_bulk(
        self,
        *,
        ottimate_company_id: int,
        vendors: typing.Optional[typing.Sequence[PostVendorsBulkRequestVendorsItem]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostVendorsBulkResponse:
        """
        Allows for the creation or updating of multiple vendors in a single request. Maximum 100 vendors per request.

        Parameters
        ----------
        ottimate_company_id : int

        vendors : typing.Optional[typing.Sequence[PostVendorsBulkRequestVendorsItem]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostVendorsBulkResponse
            Successfully created or updated

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate
        from ottimate.vendors import PostVendorsBulkRequestVendorsItem

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.vendors.post_vendors_bulk(
                ottimate_company_id=15196,
                vendors=[
                    PostVendorsBulkRequestVendorsItem(
                        erp_vendor_name="Local Produce Co",
                        erp_vendor_id="LPC-001",
                        email="orders@localproduce.com",
                        phone="+1-555-111-2222",
                        net_terms="Net 15",
                        payment_method="ach",
                        address_line1="100 Farm Road",
                        city="Springfield",
                        state="IL",
                        zipcode="62701",
                    ),
                    PostVendorsBulkRequestVendorsItem(
                        erp_vendor_name="Premium Meats LLC",
                        erp_vendor_id="PML-001",
                        email="sales@premiummeats.com",
                        payment_method="check",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_vendors_bulk(
            ottimate_company_id=ottimate_company_id, vendors=vendors, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from .types.get_accounts_id_companies_response import GetAccountsIdCompaniesResponse
from .types.get_accounts_id_locations_response import GetAccountsIdLocationsResponse
from .types.get_accounts_root_response import GetAccountsRootResponse


class RawAccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_accounts_root(
        self,
        *,
        search: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetAccountsRootResponse]:
        """
        Returns a paginated list of Ottimate accounts that the authenticated user has access to, based on partner associations linked to the OAuth client credentials.

        Parameters
        ----------
        search : typing.Optional[str]
            Search term to filter accounts by name

        limit : typing.Optional[int]
            Number of items per page

        page : typing.Optional[int]
            Page number for pagination

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAccountsRootResponse]
            Successful response with paginated account list
        """
        _response = self._client_wrapper.httpx_client.request(
            "accounts",
            method="GET",
            params={
                "search": search,
                "limit": limit,
                "page": page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAccountsRootResponse,
                    parse_obj_as(
                        type_=GetAccountsRootResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_accounts_id_companies(
        self,
        id: str,
        *,
        search: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetAccountsIdCompaniesResponse]:
        """
        Returns a paginated list of companies associated with the specified account.

        Parameters
        ----------
        id : str
            Account ID

        search : typing.Optional[str]
            Search term for company name

        limit : typing.Optional[int]
            Number of items per page

        page : typing.Optional[int]
            Page number for pagination

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAccountsIdCompaniesResponse]
            Successful response with paginated company list
        """
        _response = self._client_wrapper.httpx_client.request(
            f"accounts/{jsonable_encoder(id)}/companies",
            method="GET",
            params={
                "search": search,
                "limit": limit,
                "page": page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAccountsIdCompaniesResponse,
                    parse_obj_as(
                        type_=GetAccountsIdCompaniesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_accounts_id_locations(
        self,
        id: str,
        *,
        search: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetAccountsIdLocationsResponse]:
        """
        Returns a paginated list of locations associated with the specified account.

        Parameters
        ----------
        id : str
            Account ID

        search : typing.Optional[str]
            Search term for location name

        limit : typing.Optional[int]
            Number of items per page

        page : typing.Optional[int]
            Page number for pagination

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetAccountsIdLocationsResponse]
            Successful response with paginated location list
        """
        _response = self._client_wrapper.httpx_client.request(
            f"accounts/{jsonable_encoder(id)}/locations",
            method="GET",
            params={
                "search": search,
                "limit": limit,
                "page": page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAccountsIdLocationsResponse,
                    parse_obj_as(
                        type_=GetAccountsIdLocationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_accounts_root(
        self,
        *,
        search: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetAccountsRootResponse]:
        """
        Returns a paginated list of Ottimate accounts that the authenticated user has access to, based on partner associations linked to the OAuth client credentials.

        Parameters
        ----------
        search : typing.Optional[str]
            Search term to filter accounts by name

        limit : typing.Optional[int]
            Number of items per page

        page : typing.Optional[int]
            Page number for pagination

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAccountsRootResponse]
            Successful response with paginated account list
        """
        _response = await self._client_wrapper.httpx_client.request(
            "accounts",
            method="GET",
            params={
                "search": search,
                "limit": limit,
                "page": page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAccountsRootResponse,
                    parse_obj_as(
                        type_=GetAccountsRootResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_accounts_id_companies(
        self,
        id: str,
        *,
        search: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetAccountsIdCompaniesResponse]:
        """
        Returns a paginated list of companies associated with the specified account.

        Parameters
        ----------
        id : str
            Account ID

        search : typing.Optional[str]
            Search term for company name

        limit : typing.Optional[int]
            Number of items per page

        page : typing.Optional[int]
            Page number for pagination

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAccountsIdCompaniesResponse]
            Successful response with paginated company list
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"accounts/{jsonable_encoder(id)}/companies",
            method="GET",
            params={
                "search": search,
                "limit": limit,
                "page": page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAccountsIdCompaniesResponse,
                    parse_obj_as(
                        type_=GetAccountsIdCompaniesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_accounts_id_locations(
        self,
        id: str,
        *,
        search: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        page: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetAccountsIdLocationsResponse]:
        """
        Returns a paginated list of locations associated with the specified account.

        Parameters
        ----------
        id : str
            Account ID

        search : typing.Optional[str]
            Search term for location name

        limit : typing.Optional[int]
            Number of items per page

        page : typing.Optional[int]
            Page number for pagination

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetAccountsIdLocationsResponse]
            Successful response with paginated location list
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"accounts/{jsonable_encoder(id)}/locations",
            method="GET",
            params={
                "search": search,
                "limit": limit,
                "page": page,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetAccountsIdLocationsResponse,
                    parse_obj_as(
                        type_=GetAccountsIdLocationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

GetPurchaseOrdersRootRequestStatus = typing.Union[
    typing.Literal["pending_review", "approved", "rejected", "closed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class LineItemObject(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Line item identifier
    """

    ottimate_item_id: str = pydantic.Field()
    """
    Associated inventory item ID
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Item name from vendor
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Standardized item name
    """

    sku: typing.Optional[str] = pydantic.Field(default=None)
    """
    Stock keeping unit
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Item type classification
    """

    quantity: float = pydantic.Field()
    """
    Quantity ordered/received
    """

    unit: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unit of measurement
    """

    unit_price: float = pydantic.Field()
    """
    Price per unit
    """

    is_taxed: bool = pydantic.Field()
    """
    Whether item is subject to tax
    """

    pack_size: typing.Optional[str] = pydantic.Field(default=None)
    """
    Packaging size
    """

    uom: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unit of measure code
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawOauthClient, RawOauthClient
from .types.post_oauth_token_response import PostOauthTokenResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OauthClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOauthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawOauthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOauthClient
        """
        return self._raw_client

    def post_oauth_token(
        self,
        *,
        client_id: str,
        client_secret: str,
        scope: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOauthTokenResponse:
        """
        Exchange credentials for an access token. Supports `client_credentials` grant type.

        Parameters
        ----------
        client_id : str
            The client ID issued to the client.

        client_secret : str
            The client secret issued to the client.

        scope : typing.Optional[str]
            The scope of the access request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOauthTokenResponse
            Successful token response

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.oauth.post_oauth_token(
            client_id="example_client_id",
            client_secret="example_client_secret",
        )
        """
        _response = self._raw_client.post_oauth_token(
            client_id=client_id, client_secret=client_secret, scope=scope, request_options=request_options
        )
        return _response.data


class AsyncOauthClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOauthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawOauthClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOauthClient
        """
        return self._raw_client

    async def post_oauth_token(
        self,
        *,
        client_id: str,
        client_secret: str,
        scope: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOauthTokenResponse:
        """
        Exchange credentials for an access token. Supports `client_credentials` grant type.

        Parameters
        ----------
        client_id : str
            The client ID issued to the client.

        client_secret : str
            The client secret issued to the client.

        scope : typing.Optional[str]
            The scope of the access request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOauthTokenResponse
            Successful token response

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.oauth.post_oauth_token(
                client_id="example_client_id",
                client_secret="example_client_secret",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_oauth_token(
            client_id=client_id, client_secret=client_secret, scope=scope, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.error_response import ErrorResponse
from .types.get_dimensions_id_response import GetDimensionsIdResponse
from .types.get_dimensions_root_response import GetDimensionsRootResponse
from .types.post_dimensions_bulk_request_dimensions_item import PostDimensionsBulkRequestDimensionsItem
from .types.post_dimensions_bulk_response import PostDimensionsBulkResponse
from .types.post_dimensions_root_request_ottimate_type import PostDimensionsRootRequestOttimateType
from .types.post_dimensions_root_response import PostDimensionsRootResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDimensionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_dimensions_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        erp_type: typing.Optional[str] = None,
        detail: typing.Optional[typing.Literal["basic"]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetDimensionsRootResponse]:
        """
        Returns a paginated list of dimensions, filterable by company and type. The `detail=basic` query parameter returns a simplified response.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter dimensions by company ID

        erp_type : typing.Optional[str]
            Filter dimensions by ERP type

        detail : typing.Optional[typing.Literal["basic"]]
            Specify the level of detail in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDimensionsRootResponse]
            Successful response with paginated dimension list
        """
        _response = self._client_wrapper.httpx_client.request(
            "dimensions",
            method="GET",
            params={
                "ottimate_company_id": ottimate_company_id,
                "erp_type": erp_type,
                "detail": detail,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDimensionsRootResponse,
                    parse_obj_as(
                        type_=GetDimensionsRootResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def post_dimensions_root(
        self,
        *,
        ottimate_company_id: int,
        erp_type: str,
        ottimate_type: PostDimensionsRootRequestOttimateType,
        name: str,
        code: str,
        reference_id: str,
        erp_dimension_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostDimensionsRootResponse]:
        """
        Creates a new dimension with the provided details. The `reference_id` and `erp_dimension_id` are required for a successful creation.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID this dimension belongs to

        erp_type : str
            The dimension category label as displayed in the client's ERP system (e.g., 'Account', 'GLACCOUNT', 'Department'). This is client-specific.

        ottimate_type : PostDimensionsRootRequestOttimateType
            Ottimate's standardized dimension type code for consistent categorization.

        name : str
            Display name of the dimension

        code : str
            The GL code or short identifier for this dimension in the client's ERP system

        reference_id : str
            A unique reference identifier that tracks this dimension across all client systems (ERP, inventory management, Ottimate)

        erp_dimension_id : str
            The identifier assigned to this dimension by the client's ERP software

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostDimensionsRootResponse]
            Successfully created
        """
        _response = self._client_wrapper.httpx_client.request(
            "dimensions",
            method="POST",
            json={
                "ottimate_company_id": ottimate_company_id,
                "erp_type": erp_type,
                "ottimate_type": ottimate_type,
                "name": name,
                "code": code,
                "reference_id": reference_id,
                "erp_dimension_id": erp_dimension_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostDimensionsRootResponse,
                    parse_obj_as(
                        type_=PostDimensionsRootResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_dimensions_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetDimensionsIdResponse]:
        """
        Returns detailed information for a single dimension using its unique ID.

        Parameters
        ----------
        id : str
            The unique ID of the dimension

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDimensionsIdResponse]
            Successful response with dimension details
        """
        _response = self._client_wrapper.httpx_client.request(
            f"dimensions/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDimensionsIdResponse,
                    parse_obj_as(
                        type_=GetDimensionsIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def post_dimensions_bulk(
        self,
        *,
        ottimate_company_id: int,
        dimensions: typing.Sequence[PostDimensionsBulkRequestDimensionsItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostDimensionsBulkResponse]:
        """
        Allows for the creation or updating of multiple dimensions in a single request. The request body should contain a `ottimate_company_id` and an array of `dimensions`. Maximum 100 dimensions per request.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID these dimensions belong to

        dimensions : typing.Sequence[PostDimensionsBulkRequestDimensionsItem]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostDimensionsBulkResponse]
            Successfully created or updated
        """
        _response = self._client_wrapper.httpx_client.request(
            "dimensions/bulk",
            method="POST",
            json={
                "ottimate_company_id": ottimate_company_id,
                "dimensions": convert_and_respect_annotation_metadata(
                    object_=dimensions,
                    annotation=typing.Sequence[PostDimensionsBulkRequestDimensionsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostDimensionsBulkResponse,
                    parse_obj_as(
                        type_=PostDimensionsBulkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDimensionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_dimensions_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        erp_type: typing.Optional[str] = None,
        detail: typing.Optional[typing.Literal["basic"]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetDimensionsRootResponse]:
        """
        Returns a paginated list of dimensions, filterable by company and type. The `detail=basic` query parameter returns a simplified response.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter dimensions by company ID

        erp_type : typing.Optional[str]
            Filter dimensions by ERP type

        detail : typing.Optional[typing.Literal["basic"]]
            Specify the level of detail in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDimensionsRootResponse]
            Successful response with paginated dimension list
        """
        _response = await self._client_wrapper.httpx_client.request(
            "dimensions",
            method="GET",
            params={
                "ottimate_company_id": ottimate_company_id,
                "erp_type": erp_type,
                "detail": detail,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDimensionsRootResponse,
                    parse_obj_as(
                        type_=GetDimensionsRootResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def post_dimensions_root(
        self,
        *,
        ottimate_company_id: int,
        erp_type: str,
        ottimate_type: PostDimensionsRootRequestOttimateType,
        name: str,
        code: str,
        reference_id: str,
        erp_dimension_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostDimensionsRootResponse]:
        """
        Creates a new dimension with the provided details. The `reference_id` and `erp_dimension_id` are required for a successful creation.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID this dimension belongs to

        erp_type : str
            The dimension category label as displayed in the client's ERP system (e.g., 'Account', 'GLACCOUNT', 'Department'). This is client-specific.

        ottimate_type : PostDimensionsRootRequestOttimateType
            Ottimate's standardized dimension type code for consistent categorization.

        name : str
            Display name of the dimension

        code : str
            The GL code or short identifier for this dimension in the client's ERP system

        reference_id : str
            A unique reference identifier that tracks this dimension across all client systems (ERP, inventory management, Ottimate)

        erp_dimension_id : str
            The identifier assigned to this dimension by the client's ERP software

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostDimensionsRootResponse]
            Successfully created
        """
        _response = await self._client_wrapper.httpx_client.request(
            "dimensions",
            method="POST",
            json={
                "ottimate_company_id": ottimate_company_id,
                "erp_type": erp_type,
                "ottimate_type": ottimate_type,
                "name": name,
                "code": code,
                "reference_id": reference_id,
                "erp_dimension_id": erp_dimension_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostDimensionsRootResponse,
                    parse_obj_as(
                        type_=PostDimensionsRootResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_dimensions_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetDimensionsIdResponse]:
        """
        Returns detailed information for a single dimension using its unique ID.

        Parameters
        ----------
        id : str
            The unique ID of the dimension

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDimensionsIdResponse]
            Successful response with dimension details
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"dimensions/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDimensionsIdResponse,
                    parse_obj_as(
                        type_=GetDimensionsIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def post_dimensions_bulk(
        self,
        *,
        ottimate_company_id: int,
        dimensions: typing.Sequence[PostDimensionsBulkRequestDimensionsItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostDimensionsBulkResponse]:
        """
        Allows for the creation or updating of multiple dimensions in a single request. The request body should contain a `ottimate_company_id` and an array of `dimensions`. Maximum 100 dimensions per request.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID these dimensions belong to

        dimensions : typing.Sequence[PostDimensionsBulkRequestDimensionsItem]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostDimensionsBulkResponse]
            Successfully created or updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            "dimensions/bulk",
            method="POST",
            json={
                "ottimate_company_id": ottimate_company_id,
                "dimensions": convert_and_respect_annotation_metadata(
                    object_=dimensions,
                    annotation=typing.Sequence[PostDimensionsBulkRequestDimensionsItem],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostDimensionsBulkResponse,
                    parse_obj_as(
                        type_=PostDimensionsBulkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

GetInvoicesRootRequestStatus = typing.Union[
    typing.Literal[
        "approved",
        "incomplete",
        "archived",
        "processing",
        "needs-attention",
        "ready-for-export",
        "exported",
        "pending-approval",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawDimensionsClient, RawDimensionsClient
from .types.get_dimensions_id_response import GetDimensionsIdResponse
from .types.get_dimensions_root_response import GetDimensionsRootResponse
from .types.post_dimensions_bulk_request_dimensions_item import PostDimensionsBulkRequestDimensionsItem
from .types.post_dimensions_bulk_response import PostDimensionsBulkResponse
from .types.post_dimensions_root_request_ottimate_type import PostDimensionsRootRequestOttimateType
from .types.post_dimensions_root_response import PostDimensionsRootResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DimensionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDimensionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDimensionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDimensionsClient
        """
        return self._raw_client

    def get_dimensions_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        erp_type: typing.Optional[str] = None,
        detail: typing.Optional[typing.Literal["basic"]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetDimensionsRootResponse:
        """
        Returns a paginated list of dimensions, filterable by company and type. The `detail=basic` query parameter returns a simplified response.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter dimensions by company ID

        erp_type : typing.Optional[str]
            Filter dimensions by ERP type

        detail : typing.Optional[typing.Literal["basic"]]
            Specify the level of detail in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDimensionsRootResponse
            Successful response with paginated dimension list

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.dimensions.get_dimensions_root(
            ottimate_company_id=15196,
            erp_type="Account",
        )
        """
        _response = self._raw_client.get_dimensions_root(
            ottimate_company_id=ottimate_company_id, erp_type=erp_type, detail=detail, request_options=request_options
        )
        return _response.data

    def post_dimensions_root(
        self,
        *,
        ottimate_company_id: int,
        erp_type: str,
        ottimate_type: PostDimensionsRootRequestOttimateType,
        name: str,
        code: str,
        reference_id: str,
        erp_dimension_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostDimensionsRootResponse:
        """
        Creates a new dimension with the provided details. The `reference_id` and `erp_dimension_id` are required for a successful creation.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID this dimension belongs to

        erp_type : str
            The dimension category label as displayed in the client's ERP system (e.g., 'Account', 'GLACCOUNT', 'Department'). This is client-specific.

        ottimate_type : PostDimensionsRootRequestOttimateType
            Ottimate's standardized dimension type code for consistent categorization.

        name : str
            Display name of the dimension

        code : str
            The GL code or short identifier for this dimension in the client's ERP system

        reference_id : str
            A unique reference identifier that tracks this dimension across all client systems (ERP, inventory management, Ottimate)

        erp_dimension_id : str
            The identifier assigned to this dimension by the client's ERP software

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostDimensionsRootResponse
            Successfully created

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.dimensions.post_dimensions_root(
            ottimate_company_id=15196,
            erp_type="Department",
            ottimate_type="department",
            name="Marketing Department",
            code="MKT",
            reference_id="dept-mkt-001",
            erp_dimension_id="DEPT-MKT-001",
        )
        """
        _response = self._raw_client.post_dimensions_root(
            ottimate_company_id=ottimate_company_id,
            erp_type=erp_type,
            ottimate_type=ottimate_type,
            name=name,
            code=code,
            reference_id=reference_id,
            erp_dimension_id=erp_dimension_id,
            request_options=request_options,
        )
        return _response.data

    def get_dimensions_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDimensionsIdResponse:
        """
        Returns detailed information for a single dimension using its unique ID.

        Parameters
        ----------
        id : str
            The unique ID of the dimension

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDimensionsIdResponse
            Successful response with dimension details

        Examples
        --------
        from ottimate import Ottimate

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.dimensions.get_dimensions_id(
            id="racc_90832490382904802384",
        )
        """
        _response = self._raw_client.get_dimensions_id(id, request_options=request_options)
        return _response.data

    def post_dimensions_bulk(
        self,
        *,
        ottimate_company_id: int,
        dimensions: typing.Sequence[PostDimensionsBulkRequestDimensionsItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostDimensionsBulkResponse:
        """
        Allows for the creation or updating of multiple dimensions in a single request. The request body should contain a `ottimate_company_id` and an array of `dimensions`. Maximum 100 dimensions per request.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID these dimensions belong to

        dimensions : typing.Sequence[PostDimensionsBulkRequestDimensionsItem]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostDimensionsBulkResponse
            Successfully created or updated

        Examples
        --------
        from ottimate import Ottimate
        from ottimate.dimensions import PostDimensionsBulkRequestDimensionsItem

        client = Ottimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )
        client.dimensions.post_dimensions_bulk(
            ottimate_company_id=15196,
            dimensions=[
                PostDimensionsBulkRequestDimensionsItem(
                    erp_type="Department",
                    ottimate_type="department",
                    name="Operations Department",
                    code="OPS",
                    reference_id="dept-ops-001",
                    erp_dimension_id="DEPT-OPS-001",
                ),
                PostDimensionsBulkRequestDimensionsItem(
                    erp_type="Department",
                    ottimate_type="department",
                    name="Finance Department",
                    code="FIN",
                    reference_id="dept-fin-001",
                    erp_dimension_id="DEPT-FIN-001",
                ),
            ],
        )
        """
        _response = self._raw_client.post_dimensions_bulk(
            ottimate_company_id=ottimate_company_id, dimensions=dimensions, request_options=request_options
        )
        return _response.data


class AsyncDimensionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDimensionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDimensionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDimensionsClient
        """
        return self._raw_client

    async def get_dimensions_root(
        self,
        *,
        ottimate_company_id: typing.Optional[int] = None,
        erp_type: typing.Optional[str] = None,
        detail: typing.Optional[typing.Literal["basic"]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetDimensionsRootResponse:
        """
        Returns a paginated list of dimensions, filterable by company and type. The `detail=basic` query parameter returns a simplified response.

        Parameters
        ----------
        ottimate_company_id : typing.Optional[int]
            Filter dimensions by company ID

        erp_type : typing.Optional[str]
            Filter dimensions by ERP type

        detail : typing.Optional[typing.Literal["basic"]]
            Specify the level of detail in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDimensionsRootResponse
            Successful response with paginated dimension list

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dimensions.get_dimensions_root(
                ottimate_company_id=15196,
                erp_type="Account",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_dimensions_root(
            ottimate_company_id=ottimate_company_id, erp_type=erp_type, detail=detail, request_options=request_options
        )
        return _response.data

    async def post_dimensions_root(
        self,
        *,
        ottimate_company_id: int,
        erp_type: str,
        ottimate_type: PostDimensionsRootRequestOttimateType,
        name: str,
        code: str,
        reference_id: str,
        erp_dimension_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostDimensionsRootResponse:
        """
        Creates a new dimension with the provided details. The `reference_id` and `erp_dimension_id` are required for a successful creation.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID this dimension belongs to

        erp_type : str
            The dimension category label as displayed in the client's ERP system (e.g., 'Account', 'GLACCOUNT', 'Department'). This is client-specific.

        ottimate_type : PostDimensionsRootRequestOttimateType
            Ottimate's standardized dimension type code for consistent categorization.

        name : str
            Display name of the dimension

        code : str
            The GL code or short identifier for this dimension in the client's ERP system

        reference_id : str
            A unique reference identifier that tracks this dimension across all client systems (ERP, inventory management, Ottimate)

        erp_dimension_id : str
            The identifier assigned to this dimension by the client's ERP software

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostDimensionsRootResponse
            Successfully created

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dimensions.post_dimensions_root(
                ottimate_company_id=15196,
                erp_type="Department",
                ottimate_type="department",
                name="Marketing Department",
                code="MKT",
                reference_id="dept-mkt-001",
                erp_dimension_id="DEPT-MKT-001",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_dimensions_root(
            ottimate_company_id=ottimate_company_id,
            erp_type=erp_type,
            ottimate_type=ottimate_type,
            name=name,
            code=code,
            reference_id=reference_id,
            erp_dimension_id=erp_dimension_id,
            request_options=request_options,
        )
        return _response.data

    async def get_dimensions_id(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDimensionsIdResponse:
        """
        Returns detailed information for a single dimension using its unique ID.

        Parameters
        ----------
        id : str
            The unique ID of the dimension

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDimensionsIdResponse
            Successful response with dimension details

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dimensions.get_dimensions_id(
                id="racc_90832490382904802384",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_dimensions_id(id, request_options=request_options)
        return _response.data

    async def post_dimensions_bulk(
        self,
        *,
        ottimate_company_id: int,
        dimensions: typing.Sequence[PostDimensionsBulkRequestDimensionsItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostDimensionsBulkResponse:
        """
        Allows for the creation or updating of multiple dimensions in a single request. The request body should contain a `ottimate_company_id` and an array of `dimensions`. Maximum 100 dimensions per request.

        Parameters
        ----------
        ottimate_company_id : int
            The Ottimate company ID these dimensions belong to

        dimensions : typing.Sequence[PostDimensionsBulkRequestDimensionsItem]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostDimensionsBulkResponse
            Successfully created or updated

        Examples
        --------
        import asyncio

        from ottimate import AsyncOttimate
        from ottimate.dimensions import PostDimensionsBulkRequestDimensionsItem

        client = AsyncOttimate(
            api_key="YOUR_API_KEY",
            api_version="YOUR_API_VERSION",
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dimensions.post_dimensions_bulk(
                ottimate_company_id=15196,
                dimensions=[
                    PostDimensionsBulkRequestDimensionsItem(
                        erp_type="Department",
                        ottimate_type="department",
                        name="Operations Department",
                        code="OPS",
                        reference_id="dept-ops-001",
                        erp_dimension_id="DEPT-OPS-001",
                    ),
                    PostDimensionsBulkRequestDimensionsItem(
                        erp_type="Department",
                        ottimate_type="department",
                        name="Finance Department",
                        code="FIN",
                        reference_id="dept-fin-001",
                        erp_dimension_id="DEPT-FIN-001",
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_dimensions_bulk(
            ottimate_company_id=ottimate_company_id, dimensions=dimensions, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .get_dimensions_id_response_dimension_ottimate_type import GetDimensionsIdResponseDimensionOttimateType


class GetDimensionsIdResponseDimension(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    Unique Ottimate dimension identifier
    """

    name: str = pydantic.Field()
    """
    Display name of the dimension
    """

    erp_type: str = pydantic.Field()
    """
    The dimension category label as displayed in the client's ERP system.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The GL code or short identifier associated with this dimension.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique reference identifier that tracks this dimension across all client systems.
    """

    ottimate_type: typing.Optional[GetDimensionsIdResponseDimensionOttimateType] = pydantic.Field(default=None)
    """
    Ottimate's standardized dimension type code.
    """

    erp_dimension_id: str = pydantic.Field()
    """
    The identifier assigned to this dimension by the client's ERP software.
    """

    ottimate_company_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    The Ottimate company ID this dimension belongs to
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .variance_object_invoice_items_data_item import VarianceObjectInvoiceItemsDataItem


class VarianceObjectInvoiceItems(UniversalBaseModel):
    """
    Invoice items for this catalog entry. Only included when expand=invoice_items is specified. Returns up to 10 items with pagination metadata.
    """

    data: typing.List[VarianceObjectInvoiceItemsDataItem] = pydantic.Field()
    """
    Array of invoice items (limited to 10)
    """

    has_more: bool = pydantic.Field()
    """
    Indicates if there are more invoice items beyond the returned limit
    """

    total_count: int = pydantic.Field()
    """
    Total number of invoice items for this catalog entry
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL to fetch paginated invoice items. Only present when has_more is true.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .invoice_creation_response_upload_upload_method import InvoiceCreationResponseUploadUploadMethod


class InvoiceCreationResponseUpload(UniversalBaseModel):
    """
    Upload details and storage information
    """

    id: str = pydantic.Field()
    """
    Upload session ID
    """

    s3url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="s3_url")] = pydantic.Field(
        default=None
    )
    """
    Secure S3 storage URL
    """

    upload_method: InvoiceCreationResponseUploadUploadMethod = pydantic.Field()
    """
    Method used for upload
    """

    upload_source: typing.Optional[str] = pydantic.Field(default=None)
    """
    Source of the upload
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .variance_object_invoice_items import VarianceObjectInvoiceItems
from .variance_object_last_invoice_item import VarianceObjectLastInvoiceItem


class VarianceObject(UniversalBaseModel):
    id: int = pydantic.Field()
    """
    Catalog entry ID
    """

    ottimate_company_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Company ID
    """

    item_name: str
    sku: typing.Optional[str] = None
    upc: typing.Optional[str] = None
    pack_size: typing.Optional[str] = None
    price: typing.Optional[float] = pydantic.Field(default=None)
    """
    Catalog price
    """

    unit_price: typing.Optional[float] = pydantic.Field(default=None)
    """
    Unit price
    """

    erp_vendor_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ERP Vendor id from ERP/accounting system
    """

    case_size: typing.Optional[str] = pydantic.Field(default=None)
    """
    Pack size for unit conversion
    """

    last_invoice_item: typing.Optional[VarianceObjectLastInvoiceItem] = pydantic.Field(default=None)
    """
    Most recent invoice item for this catalog entry
    """

    invoice_items: typing.Optional[VarianceObjectInvoiceItems] = pydantic.Field(default=None)
    """
    Invoice items for this catalog entry. Only included when expand=invoice_items is specified. Returns up to 10 items with pagination metadata.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .catalog_bulk_response_errors_item import CatalogBulkResponseErrorsItem
from .catalog_bulk_response_skipped_item import CatalogBulkResponseSkippedItem


class CatalogBulkResponse(UniversalBaseModel):
    """
    Response for bulk create/update operations
    """

    success_count: int = pydantic.Field()
    """
    Total successful operations (created + updated)
    """

    created_count: int = pydantic.Field()
    """
    Number of records created
    """

    updated_count: int = pydantic.Field()
    """
    Number of records updated
    """

    error_count: int = pydantic.Field()
    """
    Number of failed operations
    """

    skipped_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of entries skipped — already exist with identical data, no changes detected
    """

    created: typing.Optional[typing.List[typing.Dict[str, typing.Any]]] = pydantic.Field(default=None)
    """
    List of created records
    """

    updated: typing.Optional[typing.List[typing.Dict[str, typing.Any]]] = pydantic.Field(default=None)
    """
    List of updated records
    """

    skipped: typing.Optional[typing.List[CatalogBulkResponseSkippedItem]] = pydantic.Field(default=None)
    """
    Entries skipped due to no field changes
    """

    errors: typing.Optional[typing.List[CatalogBulkResponseErrorsItem]] = pydantic.Field(default=None)
    """
    List of errors
    """

    prices_created: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of prices created (if applicable)
    """

    prices_updated: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of prices updated (if applicable)
    """

    allowances_created: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of allowances created (if applicable)
    """

    allowances_updated: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of allowances updated (if applicable)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .catalog_entry_object_allowances_item_allowance_type import CatalogEntryObjectAllowancesItemAllowanceType
from .catalog_entry_object_allowances_item_target_type import CatalogEntryObjectAllowancesItemTargetType


class CatalogEntryObjectAllowancesItem(UniversalBaseModel):
    """
    Catalog allowance object nested in catalog entry
    """

    id: str = pydantic.Field()
    """
    Allowance ID
    """

    allowance_type: CatalogEntryObjectAllowancesItemAllowanceType = pydantic.Field()
    """
    Type of allowance
    """

    amount: str = pydantic.Field()
    """
    Allowance amount (decimal string)
    """

    unit_allowance: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unit allowance amount (decimal string)
    """

    target: int = pydantic.Field()
    """
    Target code (10=company, 15=location_group, 20=location)
    """

    target_type: typing.Optional[CatalogEntryObjectAllowancesItemTargetType] = pydantic.Field(default=None)
    """
    Human-readable target type
    """

    ottimate_location_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Location ID
    """

    ottimate_group_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Location group ID
    """

    start_date: dt.datetime = pydantic.Field()
    """
    Allowance start date
    """

    end_date: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Allowance end date
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique key
    """

    created_date: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Record creation date
    """

    last_modified_date: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Record last modified date
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .post_dimensions_root_response_ottimate_type import PostDimensionsRootResponseOttimateType


class PostDimensionsRootResponse(UniversalBaseModel):
    version: typing.Optional[str] = None
    id: typing.Optional[str] = None
    name: typing.Optional[str] = None
    erp_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The dimension category label in client's ERP system
    """

    code: typing.Optional[str] = None
    reference_id: typing.Optional[str] = None
    ottimate_type: typing.Optional[PostDimensionsRootResponseOttimateType] = pydantic.Field(default=None)
    """
    Ottimate's standardized dimension type code
    """

    erp_dimension_id: typing.Optional[str] = None
    ottimate_company_id: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
